package ch.othello.openscep;

import java.io.IOException;
import java.io.Serializable;

import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.DERConstructedSequence;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.pkcs.CertificationRequest;
import org.bouncycastle.asn1.pkcs.RSAPrivateKeyStructure;
import org.bouncycastle.asn1.x509.RSAPublicKeyStructure;
import org.bouncycastle.asn1.x509.X509CertificateStructure;
import org.bouncycastle.asn1.x509.X509Name;

import ch.othello.openscep.internal.*;

/** Scep Client
 *
 * <pre>
 * $Id: StateData.java,v 1.1 2002/02/16 01:14:45 alani Exp $
 * $Log: StateData.java,v $
 * </pre>
 *
 * @since $Date: 2002/02/16 01:14:45 $
 * @author alani
 * @version $Revision: 1.23 $
 */

public class StateData implements Serializable {
	
	private int version;
	private int requestType;
	private byte[] certReqOrig;
	private byte[] subject;
	private byte[] pubKey;
	private byte[] privKey;
	private String communityString;
	private String url;
	private byte[] scepSubject;
	private String caCertUrl;
	
	
	public StateData( int version, int requestType,
					  String caCert,
					  DERObject certReqOrig, X509Name subject,
					  X509Name scepSubject,
					  RSAPublicKeyStructure pubKey, RSAPrivateKeyStructure privKey,
					  String communityString, String url   ) {
					  	
			Log.debug("StateData - Version " + version );
                        Log.debug("StateData - requesttype " + requestType);
                        
			setVersion( version );
			setRequestType( requestType );
			setCaCertUrl( caCert );
			//setCertReq( certReq );
			setSubject( subject );

                        setCertReqOrig( certReqOrig );
                        
			setScepSubject( scepSubject );
			
                        setPubKey( pubKey );
			setPrivKey( privKey );
			setCommunityString( communityString );
			setUrl( url );
			
	}				  



	/**
	 * Gets the certReqOrig.
	 * @return Returns a DERObject
	 */
	public DERObject getCertReqOrig() {
		try {
			return Utils.convertByteArrayToDERObject( certReqOrig );
		} catch(IOException e) {
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Sets the certReqOrig.
	 * @param certReqOrig The certReqOrig to set
	 */
	public void setCertReqOrig(DERObject certReqOrig) {
		
		try {
			this.certReqOrig = Utils.convertDERObjectToByteArray( certReqOrig.getDERObject() );
		} catch(IOException e) {
			e.printStackTrace();
		}
		
	}

	/**
	 * Gets the communityString.
	 * @return Returns a String
	 */
	public String getCommunityString() {
		return communityString;
	}

	/**
	 * Sets the communityString.
	 * @param communityString The communityString to set
	 */
	public void setCommunityString(String communityString) {
		this.communityString = communityString;
	}

	/**
	 * Gets the privKey.
	 * @return Returns a RSAPublicKeyStructure
	 */
	public RSAPrivateKeyStructure getPrivKey() {
		try {
			return new RSAPrivateKeyStructure( ((DERConstructedSequence)Utils.convertByteArrayToDERObject( privKey )));
		} catch(IOException e) {
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Sets the privKey.
	 * @param privKey The privKey to set
	 */
	public void setPrivKey(RSAPrivateKeyStructure privKey) {
		
		try {
			this.privKey = Utils.convertDERObjectToByteArray( privKey.getDERObject() );
		} catch(IOException e) {
			e.printStackTrace();
		}

	}

	/**
	 * Gets the pubKey.
	 * @return Returns a RSAPublicKeyStructure
	 */
	public RSAPublicKeyStructure getPubKey() {
		try {
			return new RSAPublicKeyStructure( ((DERConstructedSequence)Utils.convertByteArrayToDERObject( pubKey )));
		} catch(IOException e) {
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Sets the pubKey.
	 * @param pubKey The pubKey to set
	 */
	public void setPubKey(RSAPublicKeyStructure pubKey) {

		try {
			this.pubKey = Utils.convertDERObjectToByteArray( pubKey.getDERObject() );
		} catch(IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Gets the requestType.
	 * @return Returns a int
	 */
	public int getRequestType() {
		return requestType;
	}

	/**
	 * Sets the requestType.
	 * @param requestType The requestType to set
	 */
	public void setRequestType(int requestType) {
		this.requestType = requestType;
	}

	/**
	 * Gets the subject.
	 * @return Returns a X509Name
	 */
	public X509Name getSubject() {
		
		try {
			return new X509Name( ASN1Sequence.getInstance(Utils.convertByteArrayToDERObject( subject )));
		} catch(IOException e) {
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Sets the subject.
	 * @param subject The subject to set
	 */
	public void setSubject(X509Name subject) {
		try {
			this.subject = Utils.convertDERObjectToByteArray( subject.getDERObject() );
		} catch(IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Gets the version.
	 * @return Returns a int
	 */
	public int getVersion() {
		return version;
	}

	/**
	 * Sets the version.
	 * @param version The version to set
	 */
	public void setVersion(int version) {
		this.version = version;
	}

	/**
	 * Gets the url.
	 * @return Returns a String
	 */
	public String getUrl() {
		return url;
	}

	/**
	 * Sets the url.
	 * @param url The url to set
	 */
	public void setUrl(String url) {
		this.url = url;
	}

	/**
	 * Gets the caCert.
	 * @return Returns a byte[]
	 */
	public String getCaCertUrl() {

			return caCertUrl;
	}

	/**
	 * Sets the caCert.
	 * @param caCert The caCert to set
	 */
	public void setCaCertUrl(String caCertUrl) {
			this.caCertUrl = caCertUrl ;
	}

	/**
	 * Gets the scepSubject.
	 * @return Returns a byte[]
	 */
	public X509Name getScepSubject() {
		try {
			return new X509Name( ASN1Sequence.getInstance(Utils.convertByteArrayToDERObject( scepSubject )));
		} catch(IOException e) {
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Sets the scepSubject.
	 * @param scepSubject The scepSubject to set
	 */
	public void setScepSubject(X509Name scepSubject) {
		try {
			this.scepSubject = Utils.convertDERObjectToByteArray( scepSubject.getDERObject() );
		} catch(IOException e) {
			e.printStackTrace();
		}
	}

}
