package ch.othello.openscep.internal;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import org.bouncycastle.asn1.DERBitString;
import org.bouncycastle.asn1.DERConstructedSequence;
import org.bouncycastle.asn1.DERConstructedSet;
import org.bouncycastle.asn1.DEROutputStream;
import org.bouncycastle.asn1.DERPrintableString;
import org.bouncycastle.asn1.pkcs.CertificationRequest;
import org.bouncycastle.asn1.pkcs.CertificationRequestInfo;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x509.DigestInfo;
import org.bouncycastle.asn1.x509.RSAPublicKeyStructure;
import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
import org.bouncycastle.asn1.x509.X509Name;
import org.bouncycastle.crypto.AsymmetricBlockCipher;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.RuntimeCryptoException;
import org.bouncycastle.crypto.digests.MD5Digest;
import org.bouncycastle.crypto.encodings.PKCS1Encoding;
import org.bouncycastle.crypto.engines.RSAEngine;


/** generate a PKCS10 Key Request
 *  this class is currently not used
 *
 * <pre>
 * $Id: PKCS10Generate.java,v 1.5 2002/03/02 01:45:39 alani Exp $
 * $Log: PKCS10Generate.java,v $
 * Revision 1.5  2002/03/02 01:45:39  alani
 * comment, currently not used
 *
 * Revision 1.4  2002/01/10 15:51:13  alani
 * fixed PKCS10Generate, removed generation of PKCS10 in certReq
 *
 * Revision 1.3  2002/01/10 15:31:25  urut
 * pkcs10 generate
 *
 * Revision 1.2  2002/01/08 21:00:29  urut
 * added javadoc header with cvs tags to every file
 *
 * </pre>
 *
 * @since $Date: 2002/03/02 01:45:39 $
 * @author urut  
 * @version $Revision: 1.5 $
  */

public class PKCS10Generate
{
	private RSAKeyPairGen theKeyPair;

	public PKCS10Generate(RSAKeyPairGen theKeyPair)
	{
		this.theKeyPair = theKeyPair;
	}

	public DERBitString signCertificationRequestInfo (CertificationRequestInfo creqInfo) throws Exception
	{
		//now sign the whole thing
		ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
         	DEROutputStream         dOut = new DEROutputStream(bOut);
 
            	dOut.writeObject(creqInfo);
			
		Digest  digest = new MD5Digest();
		int bOutSize = bOut.size();
            	digest.update(bOut.toByteArray(), 0, bOutSize );  

		System.out.println("size of CertificateRequestInfo is " + bOutSize);
  			
		byte[]  hash = new byte[digest.getDigestSize()];
        	digest.doFinal(hash, 0);    

		//now encrypt signature with RSA
		RSAEngine eng = new RSAEngine();

		AsymmetricBlockCipher blockCiph = new PKCS1Encoding ( eng );			


        	blockCiph.init(true, theKeyPair.getPrivateKey() ); //true=for encryption   

		byte[] bytes = derEncode(hash);

		System.out.println( "inputblocksize " + blockCiph.getInputBlockSize()  ) ;

		//if a exception occures while executing processBlock: check whether the key is long enough!!
          	byte[] rsaSignature = 	blockCiph.processBlock(bytes, 0, bytes.length);   

		DERBitString signature = new DERBitString (rsaSignature );

		return ( signature );	
	}

    	public CertificationRequest generate(DERPrintableString challenge, X509Name x509name)
	{
		try {
			//todo: add unstructuredName Attribute
                        // ex: "C=CH,CN=easc.ch,O=easc,ST=ZH" );

			AlgorithmIdentifier algId = new AlgorithmIdentifier(PKCSObjectIdentifiers.rsaEncryption );

			RSAPublicKeyStructure publicKey = new RSAPublicKeyStructure( theKeyPair.getPublicModulus(), 
								theKeyPair.getPublicExponent() );
			SubjectPublicKeyInfo sPKinfo = new SubjectPublicKeyInfo (algId, publicKey.getDERObject() );
		
			//empty so far	
			DERConstructedSet attributes = new DERConstructedSet ( );
			
			DERConstructedSequence att = new DERConstructedSequence();
			att.addObject( SCEPObjectIdentifiers.challengePassword );
			att.addObject( challenge );
			attributes.addObject( att );

			CertificationRequestInfo creqInfo = new CertificationRequestInfo(x509name,sPKinfo,attributes );

			AlgorithmIdentifier signatureAlgoId = new AlgorithmIdentifier(PKCSObjectIdentifiers.md5WithRSAEncryption );
			
			CertificationRequest creq = new CertificationRequest(creqInfo, signatureAlgoId, signCertificationRequestInfo(creqInfo) );

			return (creq);
		
		} catch ( RuntimeCryptoException e)
		{
			e.printStackTrace();
			return null;
		} catch (Exception e)
		{	
			e.printStackTrace();
			return null;
		}

	}


    //return digestInfo 
    private byte[] derEncode(
        byte[]  hash)
        throws IOException
    {
        ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
        DEROutputStream         dOut = new DEROutputStream(bOut);
        DigestInfo              dInfo = new DigestInfo(new AlgorithmIdentifier(PKCSObjectIdentifiers.md5 ), hash);
 
        dOut.writeObject(dInfo);
 
        return bOut.toByteArray();
    }       

}
