package ch.othello.openscep.internal;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.asn1.util.*;
import org.bouncycastle.asn1.pkcs.*;

/**
 * <pre>
 * $Id: Utils.java,v 1.6 2002/02/22 12:38:55 urut Exp $
 * $Log: Utils.java,v $
 * Revision 1.6  2002/02/22 12:38:55  urut
 * modifications to process SUCCESS response to v2 request
 *
 * Revision 1.5  2002/02/16 01:14:45  urut
 * replaced all read-only accessed files with URL's file:///......
 * removed option caCert , only caCertURL is available now
 *
 * Revision 1.4  2002/02/15 13:21:12  urut
 * first complete implementation of scep v2
 *
 * Revision 1.3  2002/02/12 19:33:23  urut
 * replaced all println statements
 *
 * Revision 1.2  2002/02/07 17:08:50  urut
 * ability to send scep request over http added
 *
 * Revision 1.1  2002/02/07 12:40:56  urut
 * moved util-stuff to Utils.java
 * CA get implemented
 *
 * </pre>
 *
 * @since $Date: 2002/02/22 12:38:55 $
 * @author  urut
 * @version $Revision: 1.6 $
 */

public class Utils {

    public static byte[] convertDERObjectToByteArray( DERObject in) throws IOException {
		ByteArrayOutputStream bOut = new ByteArrayOutputStream();

		DEROutputStream dOut = new DEROutputStream ( bOut );
		dOut.writeObject( in );
                
                return ( bOut.toByteArray() );
    }
    
    
    public static DERObject convertByteArrayToDERObject( byte[] bArray) throws IOException {

        ByteArrayInputStream bIn = new ByteArrayInputStream( bArray );
	DERInputStream dIn = new DERInputStream(bIn);
	
        return ( dIn.readObject() );
    }
   
    
    public static void printOutByteArray( byte[] byteArray) {
        try {
                        ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
                        byteOut.write(byteArray);
                        Log.debug("content of byte array : " + byteOut.toString() );
        } catch ( Exception e) {
            e.printStackTrace();
        }
    }

    public static InputStream connectToURL( URL url) throws IOException {
           InputStream iStream = null;
                
                if ( url.getProtocol().endsWith("http") ) {
                    Log.debug("protocoll is http");
                    
                    HttpURLConnection con = (HttpURLConnection) url.openConnection();
                    con.setRequestMethod("GET");
                    con.setDoOutput(false);
                    con.connect();

                    int code = con.getResponseCode();

                    byte[] iArray = null;

                    Log.info("response code from CA get: " + code);

                    if (code == HttpURLConnection.HTTP_OK) {
			iStream = con.getInputStream();

                    } else {
			Log.error("problems getting CA Cert");
			return null;
                    }

                } else if ( url.getProtocol().equals("file") ) {
                    Log.debug("reading from file stream");
                    iStream = url.openStream();                    
                }
           
           return (iStream);
    }
    
    public static OutputStream writeToURL( URL url) throws IOException {
           OutputStream oStream = null;
                
                if ( url.getProtocol().endsWith("http") ) {
                    Log.error("can't write to http");
                } else if ( url.getProtocol().equals("file") ) {
                    Log.debug("writing to file stream");
                    
                    Log.debug("path : " + url.getPath() + " file " + url.getFile() );
                    
                    //because URL doesn't support outputstream
                    oStream = new FileOutputStream(new File(url.getFile()) ) ;
                } else {
                    Log.error("invalid protocoll to write to");
                }
           
           return (oStream);
    }

    
}
