package ch.othello.openscep.servlet;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.URLDecoder;
import java.text.DateFormat;
import java.text.MessageFormat;
import java.util.Date;

import Acme.Serve.servlet.ServletException;
import Acme.Serve.servlet.ServletOutputStream;
import Acme.Serve.servlet.http.Cookie;
import Acme.Serve.servlet.http.HttpServletRequest;
import Acme.Serve.servlet.http.HttpServletResponse;
import ch.othello.openscep.ScepClient;
import ch.othello.openscep.ScepError;
import ch.othello.openscep.ScepPending;
import ch.othello.openscep.StateData;
import ch.othello.openscep.internal.Log;

/** Scep Http Servlet for a Netscape Certification Request 
 *
 * <pre>
 * $Id: BasicPollServlet.java,v 1.7 2002/03/02 01:40:15 alani Exp $
 * $Log: BasicPollServlet.java,v $
 * Revision 1.7  2002/03/02 01:40:15  alani
 * added redirect before installation on netscape browsers
 * code cleaning
 *
 * Revision 1.6  2002/02/27 04:39:35  alani
 * url decoding the cookie
 * handling of language setting by url param
 *
 * Revision 1.5  2002/02/24 23:54:32  alani
 * added decorator functionality
 *
 * Revision 1.4  2002/02/24 22:29:01  alani
 * added concatenation of the different cookies, due to
 * mysterious reasons seems not to work on msie
 *
 * Revision 1.3  2002/02/23 00:02:47  alani
 * refactoring the servlet classes, fixed some errors in BasicRequest & BasicPoll Servlets
 *
 * Revision 1.2  2002/02/22 07:40:05  alani
 * fixed & improved Polling for requests
 *
 * Revision 1.1  2002/02/14 17:18:17  alani
 * fixed NetscapeRequest, first try at polling
 *
 * Revision 1.1  2002/01/24 13:14:54  alani
 * wrote HttpClient
 *
 * </pre>
 *
 * @since $Date: 2002/03/02 01:40:15 $
 * @author  
 * @version $Revision: 1.7 $
 */

public abstract class BasicPollServlet extends BasicServlet {

	abstract protected void browserInstallCertificate(HttpServletResponse response)
		throws IOException;

	public void sendDonePage(HttpServletResponse response) throws IOException {
		ServletOutputStream p = response.getOutputStream();

		sendHeader(p, res.getString("certInstalled"));
		p.println(
			"<CENTER><H1>" + res.getString("certInstalledLong") + "</H1></CENTER>");
		sendFooter(p);

	}

	public void sendNotAccepted(HttpServletResponse response) throws IOException {
		ServletOutputStream p = response.getOutputStream();

		sendHeader(p, res.getString("certNotInstalled"));
		p.println(
			"<CENTER><H1>" + res.getString("certNotInstalledLong") + "</H1></CENTER>");
		sendFooter(p);
	}

	abstract public void browserInstallSuccess(
		HttpServletResponse response,
		ServletOutputStream p, String cert) throws IOException;

	public void service(HttpServletRequest req, HttpServletResponse response)
		throws ServletException, IOException {

		ServletOutputStream p = response.getOutputStream();

		initLocale(req);
		loadConfig();

		if (req.getParameter("a") != null) {
			if (req.getParameter("a").equals("done"))
				sendDonePage(response);
			else
				sendNotAccepted(response);
		} else if (req.getParameter("cert") != null) {
			browserInstallSuccess(response, p, req.getParameter("cert") );
		} else {

			Cookie[] cookies = req.getCookies();
			StringBuffer state = new StringBuffer();
			for (int i = 0; i < cookies.length; i++) {
				int n = cookies[i].getName().indexOf("scepState");
				if (n >= 0) {
					state.append(cookies[i].getValue());
				}
			}

			if (state != null) {
				String theState = state.toString();

				try {
					byte[] bytes =
						org.bouncycastle.util.encoders.Base64.decode(URLDecoder.decode(theState));

					ByteArrayInputStream bIn = new ByteArrayInputStream(bytes);
					ObjectInputStream iis = new ObjectInputStream(bIn);

					scep = new ScepClient();
					scep.setStateData((StateData) iis.readObject());
					scep.Execute("certRequest");

					browserInstallCertificate(response);

				} catch (ClassNotFoundException ex) {
					Log.error("Class ScepClient not found when deserializing the Client State");
					sendErrorPage(
						response,
						new ScepError("Class ScepClient not found when deserializing the Client State"));
					ex.printStackTrace();
				} catch (ScepError ex) {
					Log.error("ScepError encountered when polling");
					sendErrorPage(response, ex);
					ex.printStackTrace();
				} catch (ScepPending ex) {

					response.setHeader(
						"Refresh",
						options.getProperty("pollTimeout") + ";URL=" + req.getServletPath());
					response.setStatus(HttpServletResponse.SC_OK);
					response.setContentType("text/html");

					sendHeader(p, res.getString("certificateEnrollment"));
					p.println("<CENTER>");

					String now =
						DateFormat.getTimeInstance(DateFormat.DEFAULT, locale).format(new Date());
					p.println("<H1>" + res.getString("enrollmentPendingTitle") + "</H1>");
					p.println(
						"<H3>"
							+ MessageFormat.format(
								res.getString("enrollmentPendingInfo"),
								new Object[] { options.getProperty("pollTimeout"), now })
							+ "</H3>");
					p.println("<P>" + res.getString("enrollmentPendingBookmark") + "</P>");

					p.println("</CENTER>");
					sendFooter(p);
				} catch (Exception ex) {
					ex.printStackTrace();
				}

			} else {
				sendErrorPage(response, new ScepError("the SCEP State Cookie is not set"));
			}
		}

		p.flush();
		p.close();

	}

}