package ch.othello.openscep.servlet;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.zip.GZIPOutputStream;

import Acme.Serve.servlet.ServletException;
import Acme.Serve.servlet.ServletOutputStream;
import Acme.Serve.servlet.http.HttpServletRequest;
import Acme.Serve.servlet.http.HttpServletResponse;
import ch.othello.openscep.ScepClient;
import ch.othello.openscep.ScepError;
import ch.othello.openscep.ScepPending;

/** Scep Http Servlet for a Netscape Certification Request 
 *
 * <pre>
 * $Id: BasicRequestServlet.java,v 1.11 2002/03/02 01:40:49 alani Exp $
 * $Log: BasicRequestServlet.java,v $
 * Revision 1.11  2002/03/02 01:40:49  alani
 * made this class abstract
 * code cleaning
 *
 * Revision 1.10  2002/02/27 04:12:26  alani
 * chunked cookie is now only 500 bytes
 * add expires= to cookie
 *
 * Revision 1.9  2002/02/24 23:55:32  alani
 * added decorator functionality, it depends on the forceChallenge property now if the user is asked for a challenge password
 *
 * Revision 1.8  2002/02/24 22:27:22  alani
 * added chunking of state data into several cookies if the data
 * is bigger than 4000 bytes
 *
 * Revision 1.7  2002/02/23 23:05:38  alani
 * renamed SPKAC field to CERTREQ
 * made redirect to pollservlet independent of the browser type
 *
 * Revision 1.6  2002/02/23 00:02:47  alani
 * refactoring the servlet classes, fixed some errors in BasicRequest & BasicPoll Servlets
 *
 * Revision 1.5  2002/02/22 07:40:05  alani
 * fixed & improved Polling for requests
 *
 * Revision 1.4  2002/02/22 01:30:42  alani
 * correctly set the attributes caCert, pubKey, privKey, subject, subjectSCEP and communityString
 *
 * Revision 1.3  2002/02/14 21:45:37  alani
 * made servlet output internationalized
 * added redirection with browser detection for unknown urls
 *
 * Revision 1.2  2002/02/14 17:18:17  alani
 * fixed NetscapeRequest, first try at polling
 *
 * Revision 1.1  2002/02/13 12:00:05  alani
 * web servlets refactored
 *
 * Revision 1.1  2002/01/24 13:14:54  alani
 * wrote HttpClient
 *
 * </pre>
 *
 * @since $Date: 2002/03/02 01:40:49 $
 * @author  
 * @version $Revision: 1.11 $
 */

public abstract class BasicRequestServlet extends BasicServlet {

	public void service(HttpServletRequest req, HttpServletResponse response)
		throws ServletException, IOException {
			
		// FIXME, remove this try catch when finished with initial devel
		try {		
		initLocale( req );
		loadConfig();
		
		boolean forceChallenge = false;
		String chall = options.getProperty("forceChallenge");
		if ( chall != null && chall.equalsIgnoreCase("true") )
			forceChallenge = true;

		if (req.getParameter("CERTREQ") == null && req.getParameter("SPKAC") == null ) {
			if ( forceChallenge && req.getParameter("challenge") == null )
				askForChallenge(req, response);
			else
				sendEnrollmentForm(req, response);
		} else
			sendCertificateRequest(req, response);
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}

	public void askForChallenge(
		HttpServletRequest req,
		HttpServletResponse response)
		throws ServletException, IOException {

		response.setStatus(HttpServletResponse.SC_OK);
		response.setContentType("text/html");
		ServletOutputStream p = response.getOutputStream();
		String action = req.getServletPath().substring(0, 
					    req.getServletPath().lastIndexOf('/') + 1);


		sendHeader(p, res.getString("askChallengeKeygen"));
		p.println("<CENTER>");
		p.println("<H2>" + res.getString("askChallengeKeygen") + "</H2>");
		p.println("<FORM NAME=\"GenerateForm\" ACTION=\""+action+"request\">");
		p.println("<TABLE>");
		p.println("<TR><TD>" + res.getString("challengePassword") + "</TD><TD>");
		p.println(
			"<INPUT TYPE=\"TEXT\" NAME=\"challenge\" VALUE=\"" + options.getProperty("challenge") + "\" SIZE=64>");
		p.println("</TD></TR>");
		p.println("</TABLE>");
		p.println("<INPUT TYPE=\"SUBMIT\" NAME=\"SUBMIT\">");
		p.println("</FORM>");

		p.println("</BODY></HTML>");
		p.flush();
		p.close();
	}

	public void textBox(ServletOutputStream p, String name, int size)
		throws IOException {

		try {
			String defname = name + "_Default";
			if (options.getProperty(defname) != null
				&& options.getProperty(defname).equals("true")) {
				p.println(options.getProperty(name));
				p.println(
					"<INPUT TYPE=\"HIDDEN\" NAME=\""
						+ name
						+ "\" VALUE=\""
						+ options.getProperty(name)
						+ "\">");
			} else {
				p.println(
					"<INPUT TYPE=\"TEXT\" NAME=\""
						+ name
						+ "\" VALUE=\""
						+ options.getProperty(name)
						+ "\" SIZE="
						+ size
						+ ">");
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	abstract public void sendFormHeader(ServletOutputStream p) throws IOException;

	abstract public void sendFormFooter(HttpServletRequest req, ServletOutputStream p)
		throws IOException;

	public void sendEnrollmentForm(
		HttpServletRequest req,
		HttpServletResponse response)
		throws ServletException, IOException {

		loadConfig();

		response.setStatus(HttpServletResponse.SC_OK);
		response.setContentType("text/html");
		ServletOutputStream p = response.getOutputStream();


		sendHeader(p, res.getString("certificateEnrollment") );		
		sendFormHeader(p);

		p.println("<TABLE>");
		p.println("<TR><TD>" + res.getString("commonName") + "</TD><TD>");
		textBox(p, "commonName", 64);
		p.println("</TD></TR>");
		p.println("<TR><TD>" + res.getString("email") + "</TD><TD>");
		textBox(p, "emailAddress", 40);
		p.println("</TD></TR>");
		p.println("<TR><TD>" + res.getString("organization") + "</TD><TD>");
		textBox(p, "organization", 40);
		p.println("</TD></TR>");
		p.println("<TR><TD>" + res.getString("organizationUnit") + "</TD><TD>");
		textBox(p, "orgUnit", 40);
		p.println("</TD></TR>");
		p.println("<TR><TD>" + res.getString("locality") + "</TD><TD>");
		textBox(p, "locality", 40);
		p.println("</TD></TR>");
		p.println("<TR><TD>" + res.getString("state") + "</TD><TD>");
		textBox(p, "state", 20);
		p.println("</TD></TR>");
		p.println("<TR><TD>" + res.getString("country") + "</TD><TD>");
		textBox(p, "country", 20);
		p.println("</TD></TR>");
		p.println("</TABLE>");
		p.println(
			"<INPUT TYPE=\"HIDDEN\" NAME=\"VERSION\" VALUE=\""
				+ req.getParameter("VERSION")
				+ "\">");

		sendFormFooter(req, p);
		
		sendFooter(p);
		
		p.flush();
		p.close();

	}

	public void sendPendingPage(
		HttpServletResponse response,
		ServletOutputStream p)
		throws IOException {
		response.setContentType("text/html");
		// we should redirect to the polling page
	}

	protected String getParam(HttpServletRequest req, String name) {
		if ( req.getParameter(name) != null )
			return URLDecoder.decode(req.getParameter(name));
		else
			return "";
	}

	public void sendCertificateRequest(
		HttpServletRequest req,
		HttpServletResponse response)
		throws ServletException, IOException {

		try {

			loadConfig();

			scep = new ScepClient();

			scep.setUrl(options.getProperty("url"));



			scep.loadCA( options.getProperty("caCertUrl") );
			scep.loadPrivateKey(new URL(options.getProperty("privateKey")));
			scep.loadPublicKey(new URL(options.getProperty("publicKey")));
			scep.setCommunityString(options.getProperty("communityString"));
			scep.setSubjectScep(options.getProperty("proxySubject"));

			scep.setScepVersion(scep.VERSION2);

			browserCertificateRequest(req);

			response.setStatus(HttpServletResponse.SC_OK);

			ServletOutputStream p = response.getOutputStream();

			try {

				scep.Execute("certRequest");

				installCertificate(response, p);
			} catch (ScepPending e) {
				//serialize the scepClient and set a Cookie
				ByteArrayOutputStream bOut = new ByteArrayOutputStream();
				ObjectOutputStream oos = new ObjectOutputStream( bOut );
				oos.writeObject(scep.getStateData());
				oos.close();

				byte[] outBytes =
					org.bouncycastle.util.encoders.Base64.encode(bOut.toByteArray());
						
				String out = URLEncoder.encode( new String( outBytes) );
				SimpleDateFormat formatter = new SimpleDateFormat( "EEEE, d-MMM-yyyy hh:mm:ss z" );
				Calendar cal = formatter.getCalendar();
				cal.setTimeZone( TimeZone.getTimeZone("GMT"));
				cal.setTime( new Date() );
				cal.add( Calendar.DAY_OF_MONTH, 30 );
				String expire = formatter.format( cal.getTime() );

				int i = 0;
				int pos = 0;
				int end = 0;
				while ( pos < out.length() ) {
					if ( pos + 500 > out.length() )
						end = out.length();
					else
						end = pos + 500;
			
					// set the cookie				
					response.setHeader( "Set-Cookie", "scepState" + i + "=" + out.substring( pos,end) + "; expires=" + expire + ";" );
					i++;
					pos += 500;
				}
				
				String base =
					req.getServletPath().substring(0, req.getServletPath().lastIndexOf('/') + 1);

				//redirect to PollingPage with Refresh
				response.sendRedirect(base + "poll");
			} catch (ScepError e) {
				// send an error Page to the client
				response.setContentType("text/html");
				// here we send the private Key
				p.println("<HTML><HEAD>");
				p.println("<TITLE>" + res.getString("errorEnrollmentTitle") + "</TITLE>");
				p.println("</HEAD><BODY><CENTER>");
				p.println("<H2>" + res.getString("errorEnrollmentSentence") + "</H2>");
				p.println("<P>" + res.getString("contactSysAdmin"));
				p.println("<P>" + res.getString("errorMessage") + e.getMessage());
				p.println("</BODY></HTML>");

			} finally {
				p.flush();
				p.close();
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	protected void installCertificate(
		HttpServletResponse response,
		ServletOutputStream p)
		throws IOException {
		response.setContentType("text/html");
		// here we send the private Key
		p.println("<HTML><HEAD>");
		p.println("<TITLE>PrivateKey is now installed</TITLE>");
		p.println("</HEAD><BODY><CENTER>");
		p.println("<H2>Enrollment sent to SCEP Server</H2>");
		p.println("</BODY></HTML>");
	}

	abstract protected void browserCertificateRequest(HttpServletRequest req);
}