package ch.othello.openscep.servlet;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Locale;
import java.util.Properties;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

import Acme.Serve.servlet.ServletException;
import Acme.Serve.servlet.ServletOutputStream;
import Acme.Serve.servlet.http.HttpServlet;
import Acme.Serve.servlet.http.HttpServletRequest;
import Acme.Serve.servlet.http.HttpServletResponse;
import ch.othello.openscep.ScepClient;

/** Basic Servlet for the Scep Proxy
 *
 * <pre>
 * $Id: BasicPollServlet.java,v 1.2 2002/02/22 07:40:05 alani Exp $
 * $Log: BasicPollServlet.java,v $
 *
 * </pre>
 *
 * @since $Date: 2002/02/22 07:40:05 $
 * @author  
 * @version $Revision: 1.2 $
 */

public class BasicServlet extends HttpServlet {
	
	static String configFile = "";
	static Properties options = null;
	protected ResourceBundle res = null;
	protected Locale locale;
	protected ScepClient scep;
	protected String decoHead = null;
	protected String decoFoot = null;
	
	public void service(HttpServletRequest req, HttpServletResponse response)
		throws ServletException, IOException {}
		
		
	public static void setConfigFile(String file) {
		configFile = file;
	}
	
	protected void loadConfig() {

		options = new Properties();
		InputStream in;
			
		try {
			File cFile = new File(configFile);
			in = new FileInputStream(cFile);
		} catch ( FileNotFoundException ex ) {
			in = getClass().getResourceAsStream( "scepProxyConfig" );
		}

		try {
			options.load(in);
		} catch (IOException e) {
			e.printStackTrace();
		}

		loadDecorator();
	}

	public void sendErrorPage( HttpServletResponse response, Exception e ) throws IOException {
				// send an error Page to the client
				response.setContentType("text/html"); 
				ServletOutputStream p = response.getOutputStream();
				// here we send the private Key
				sendHeader(p, res.getString("errorEnrollmentTitle") );
				p.println("<CENTER>");
				p.println("<H2>" + res.getString("errorEnrollmentSentence") + "</H2>");
				p.println("<P>" + res.getString("contactSysAdmin"));
				p.println("<P>" + res.getString("errorMessage") + e.getMessage());
				p.println("</CENTER>");
				sendFooter(p);
	}

	public void initLocale( HttpServletRequest req ) {
	

        if ( req.getParameter("lang") != null ) {
            locale = new Locale( req.getParameter("lang"), "" );
        } else if ( req.getParameter("Accept-Language") != null ) {
            // primitive content negotiation .... could be improved alot !!!
            StringTokenizer tk = new StringTokenizer(req.getHeader("Accept-Language"), ",");
            String lg = tk.nextToken();
            if ( lg != null ) 
            	locale = new Locale(lg, "" );
            else
            	locale = new Locale( "en", "" );
        } else
        	locale = new Locale("", "");

		res = PropertyResourceBundle.getBundle("HTTPResources", locale );		
	}

	private void loadDecorator() {

		if ( decoHead == null ) {				
		try {
			String deco = "";
			String decoFile = options.getProperty("decorator");
			if ( decoFile != null && decoFile.length() > 0 ) {
				
				Reader reader = null;
				try {
					reader = new FileReader(decoFile);
				} catch ( FileNotFoundException e ) {
					reader = new InputStreamReader( 
						getClass().getResourceAsStream( decoFile ));
				}

				BufferedReader in = new BufferedReader( reader );
				String line;
				while(( line = in.readLine()) != null ) {
					deco += line;
				}
				
				int p = deco.indexOf("{SCEPCONTENT}");
				if ( p >= 0 ) {
					decoHead = deco.substring(0, p );
					decoFoot = deco.substring(p+13, deco.length() );
				}
			}
		} catch(FileNotFoundException e) {
			e.printStackTrace();
		} catch(IOException e) {
			e.printStackTrace();
		}
		}
	}

	public void sendHeader( ServletOutputStream p, String title ) throws IOException {

		if ( decoHead != null ) {
			int tpos = decoHead.indexOf( "{SCEPTITLE}" );
			StringBuffer dh = new StringBuffer( decoHead );	
			dh.replace( tpos, tpos+11, title );
			p.print( dh.toString() );		
		} else {
			p.println("<HTML><HEAD>");
			p.println("<TITLE>" + title + "</TITLE>");
			p.println("</HEAD><BODY>");
		}
		
	}
	
	public void sendFooter( ServletOutputStream p ) throws IOException {
		
		if ( decoFoot != null ) {
			p.print( decoFoot );
		} else {
			p.println("</BODY>");
			p.println("</HTML>");
		}
			
	}
	
}
