package ch.othello.openscep.servlet;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.net.URLDecoder;

import Acme.Serve.servlet.ServletException;
import Acme.Serve.servlet.ServletOutputStream;
import Acme.Serve.servlet.http.Cookie;
import Acme.Serve.servlet.http.HttpServletRequest;
import Acme.Serve.servlet.http.HttpServletResponse;
import ch.othello.openscep.internal.Log;

/** Scep Http Servlet for a Netscape Certification Request 
 *
 * <pre>
 * $Id: NetscapeRequestServlet.java,v 1.10 2002/03/02 01:42:56 alani Exp $
 * $Log: NetscapeRequestServlet.java,v $
 * Revision 1.10  2002/03/02 01:42:56  alani
 * fixed & improved cert installation
 *
 * Revision 1.9  2002/02/24 23:59:40  alani
 * renamed functions sendHeader, sendFooter to sendFormXXX
 * was needed because of decorator functionality
 * load default challenge from config file
 *
 * Revision 1.8  2002/02/23 23:04:05  alani
 * added extraction of the email adress for distinguished name
 * renamed SPKAC field to CERTREQ
 *
 * Revision 1.7  2002/02/23 00:02:47  alani
 * refactoring the servlet classes, fixed some errors in BasicRequest & BasicPoll Servlets
 *
 * Revision 1.6  2002/02/22 07:40:05  alani
 * fixed & improved Polling for requests
 *
 * Revision 1.5  2002/02/22 01:30:33  alani
 * correctly set the attributes caCert, pubKey, privKey, subject, subjectSCEP and communityString
 *
 * Revision 1.4  2002/02/14 21:45:37  alani
 * made servlet output internationalized
 * added redirection with browser detection for unknown urls
 *
 * Revision 1.3  2002/02/14 17:18:17  alani
 * fixed NetscapeRequest, first try at polling
 *
 * Revision 1.2  2002/02/13 12:00:05  alani
 * web servlets refactored
 *
 * Revision 1.1  2002/01/24 13:14:54  alani
 * wrote HttpClient
 *
 * </pre>
 *
 * @since $Date: 2002/03/02 01:42:56 $
 * @author  
 * @version $Revision: 1.10 $
 */

public class NetscapeRequestServlet extends BasicRequestServlet {

	public void askForChallenge(
		HttpServletRequest req,
		HttpServletResponse response)
		throws ServletException, IOException {

		response.setStatus(HttpServletResponse.SC_OK);
		response.setContentType("text/html");
		ServletOutputStream p = response.getOutputStream();
		
		sendHeader( p, res.getString( "askChallengeKeygen" ));
		p.println("<CENTER>");
		p.println("<H2>" + res.getString("askChallengeKeygen") +"</H2>");
		p.println("<FORM NAME=\"GenerateForm\" ACTION=\"/netscape/request\">");
		p.println("<TABLE>");
		p.println("<TR><TD>" + res.getString("challengePassword") + "</TD><TD>");
		p.println(
			"<INPUT TYPE=\"TEXT\" NAME=\"challenge\" VALUE=\"" + options.getProperty("challenge") + "\" SIZE=64>");
		p.println("</TD></TR>");
		p.println("</TABLE>");
		p.println("<INPUT TYPE=\"SUBMIT\" NAME=\"SUBMIT\">");
		p.println("</FORM>");
		p.println("</CENTER>");

		sendFooter(p);
		p.flush();
		p.close();
	}

	public void sendFormHeader(ServletOutputStream p) throws IOException {

		p.println("<SCRIPT LANGUAGE=\"JavaScript\"");
		p.println("function genReq() {}");
		p.println("</SCRIPT>");
		p.println("<CENTER>");
		p.println("<H2>" + res.getString("certificateEnrollment") + "</H2>");
		p.println("<FORM NAME=\"GenerateForm\" ACTION=\"/netscape/request\" >");
	}

	public void sendFormFooter(HttpServletRequest req, ServletOutputStream p)
		throws IOException {

		String challengePassword = req.getParameter("challenge");
		if ( challengePassword == null ) challengePassword = "";
		
		p.println("<KEYGEN NAME=\"SPKAC\" CHALLENGE=\"" + challengePassword + "\">");

		p.println("<INPUT TYPE=\"SUBMIT\" NAME=\"SUBMIT\" VALUE=\"" + res.getString("next") + "\">");
		p.println("</FORM>");
		p.println("</CENTER>");
	}

	protected void browserCertificateRequest(HttpServletRequest req) {
		StringBuffer dn = new StringBuffer();

		dn.append("CN="); // common name
		dn.append(URLDecoder.decode(req.getParameter("commonName")));

		dn.append(",OU="); // organization unit
		if (options.getProperty("orgUnit_Default") != null
			&& options.getProperty("orgUnit_Default").equals("true"))
			dn.append(getParam(req, "orgUnit"));
		else
			dn.append(options.getProperty("orgUnit"));

		dn.append(",O="); // organization
		if (options.getProperty("organization_Default") != null
			&& options.getProperty("organization_Default").equals("true"))
			dn.append(getParam(req, "organization"));
		else
			dn.append(options.getProperty("organization"));

		dn.append(",L="); // Stadt
		if (options.getProperty("locality_Default") != null
			&& options.getProperty("locality_Default").equals("true"))
			dn.append(getParam(req, "locality"));
		else
			dn.append(options.getProperty("locality"));

		dn.append(",ST="); // state
		if (options.getProperty("state_Default") != null
			&& options.getProperty("state_Default").equals("true"))
			dn.append(getParam(req, "state"));
		else
			dn.append(options.getProperty("state"));

		dn.append(",C="); // country
		if (options.getProperty("country_Default") != null
			&& options.getProperty("country_Default").equals("true"))
			dn.append(getParam(req, "country"));
		else
			dn.append(options.getProperty("country"));

		if ( getParam(req,"emailAddress").length() > 0 ) {
			dn.append(",EmailAddress="); // country
			dn.append(getParam(req, "country"));
		}

		scep.setSubject(dn.toString());
		scep.setSubjectScep( dn.toString() );

		String spkac = getParam(req, "SPKAC");

		Log.debug(spkac);
		String base64Input = new String();
				
		// remove the linebreaks 
		BufferedReader dStream =
			new BufferedReader(new StringReader( spkac ));

		try {
			String line = dStream.readLine();
			while ( line != null ) {
				
				base64Input += line;
				line = dStream.readLine();
			}
		} catch(IOException e) {
			e.printStackTrace();
		}
		
		
		// set request type to spkac
		scep.setRequestType( scep.TYPE_SPKAC );
		scep.setCertReqOrig(base64Input);

	}

	protected void installCertificate(
		HttpServletResponse response,
		ServletOutputStream p)
		throws IOException {

		response.setContentType("application/x-x509-user-cert");
		String cert = scep.getBase64SignedEndEntityCertificate();
		response.setContentLength(cert.length());

		p.print(cert);
	}

}